
// *****************************************************************************
//
// source:    AC.C
//
// type:      source code
//
// created:   26 Feb 2000
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: AC routines
//
// changes:
// 26 Feb 2000 (AG) creations
// 21 Mar 2000 (AG) check presence of component for AC_flag()
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// ONLINE includes
#include "component.h"

// Arte includes
#include "alca/AC.h"
#include "farm/farm.hh"

// AC externals
extern int AC_SVD;
extern int AC_ITR;
extern int AC_HIPT;
extern int AC_OTR;
extern int AC_RICH;
extern int AC_TRD;
extern int AC_ECAL;
extern int AC_MUON;

extern int AC_CORR;
extern int AC_GLOB;

// prototypes
extern "C" void kugetc_( char *string, int *len, int len1 );

//
// AC_flag() by component ID
//
extern "C" int
AC_flag( int COMP_XYZ )
{
  int rc = 0;

  if( ( COMP_XYZ == COMP_SVD  ) && AC_SVD  && faComp( COMP_M_SVD  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ITR  ) && AC_ITR  && faComp( COMP_M_ITR  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_HIPT ) && AC_HIPT && faComp( COMP_M_HIPT ) ) rc = 1;
  if( ( COMP_XYZ == COMP_OTR  ) && AC_OTR  && faComp( COMP_M_OTR  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RICH ) && AC_RICH && faComp( COMP_M_RICH ) ) rc = 1;
  if( ( COMP_XYZ == COMP_TRD  ) && AC_TRD  && faComp( COMP_M_TRD  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ECAL ) && AC_ECAL && faComp( COMP_M_ECAL ) ) rc = 1;
  if( ( COMP_XYZ == COMP_MUON ) && AC_MUON && faComp( COMP_M_MUON ) ) rc = 1;

  if( ( COMP_XYZ == COMP_CORR ) && AC_CORR ) rc = 1;
  if( ( COMP_XYZ == COMP_GLOB ) && AC_GLOB ) rc = 1;

  //cout << "AC_flag(): COMP_XYZ="  << COMP_XYZ << " LVL_M_ABC=" << LVL_M_ABC << " rc=" << rc << endl;

  return rc;
}

//
// ac_comp
//
extern "C" void
ac_comp_()
{   
  //cout << "ac_comp_()" << endl;

  //
  // read in subdetector
  //
  int len  = 0;
  int len1 = 4;
  char input[7];
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *subdetector  = (char *) malloc( len + 1 );
  strncpy(subdetector, input, len);
  subdetector[len] = '\0';
  //cout << "ac_comp_(): subdetector=" << subdetector << endl;
  
  // loop
  for( int i=0; i<len; i++ ){
    subdetector[i] = toupper( subdetector[i] );
  }

  // check
  if( strncmp( subdetector, "SVD",  3 ) && 
      strncmp( subdetector, "VDS",  3 ) &&
      strncmp( subdetector, "ITR",  3 ) && 
      strncmp( subdetector, "HIPT", 4 ) &&
      strncmp( subdetector, "OTR",  3 ) && 
      strncmp( subdetector, "RICH", 4 ) &&
      strncmp( subdetector, "TRD",  3 ) && 
      strncmp( subdetector, "ECAL", 4 ) &&
      strncmp( subdetector, "MUON", 3 ) && 
      strncmp( subdetector, "SLT",  3 ) &&
      strncmp( subdetector, "TLT",  3 ) &&
      strncmp( subdetector, "FARM", 4 ) && 
      strncmp( subdetector, "CORR", 4 ) &&
      strncmp( subdetector, "GLOB", 4 )    ){
    cout << "ac_comp_(): ********* AC/SUBD: unknown subdetector: "
         << subdetector << endl;
  }
  else {
    //cout << "AC/SUBD: " << subdetector << endl;
  }
  
  //
  // read in status
  //
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *status  = (char *) malloc( len + 1 );
  strncpy( status, input, len );
  status[len] = '\0';
  //cout << "ac_comp_(): status=" << status << endl;
  
  // convert to upper case letters
  if( len>0 ){
    for( int i=0; i<len; i++ ){
      status[i] = toupper( status[i] );
    }
  }  

  // check
  if( strncmp( status, "ON",  2 ) &&
      strncmp( status, "OFF", 3 )    ){
    cout << "ac_comp_(): ********* AC/SUBD: unknown subdetector status="
	 << status << endl;
  }
  else {
    //cout << "AC/SUBD: " << status << endl;
  }

  //
  // set AC status
  //
  int level = 1;
  if (!strncmp( status, "OFF", 3 ) ) level = 0;

  if( !strncmp( subdetector, "SVD",  3 ) ) AC_SVD  = level;
  if( !strncmp( subdetector, "VDS",  3 ) ) AC_SVD  = level;
  if( !strncmp( subdetector, "ITR",  3 ) ) AC_ITR  = level;
  if( !strncmp( subdetector, "HIPT", 4 ) ) AC_HIPT = level;
  if( !strncmp( subdetector, "OTR",  3 ) ) AC_OTR  = level;
  if( !strncmp( subdetector, "RICH", 4 ) ) AC_RICH = level;
  if( !strncmp( subdetector, "TRD",  3 ) ) AC_TRD  = level;
  if( !strncmp( subdetector, "ECAL", 4 ) ) AC_ECAL = level;
  if( !strncmp( subdetector, "MUON", 4 ) ) AC_MUON = level;
  if( !strncmp( subdetector, "CORR", 4 ) ) AC_CORR = level;
  if( !strncmp( subdetector, "GLOB", 4 ) ) AC_GLOB = level;

  // print
  cout << "AC/SUBD: " << subdetector << " " << status << " " << level << endl;
}

//
// print
//
void
AC_print( const char *txt )
{
  cout << "AC_print(): Called from " << txt << ":" << endl;
  cout << "            AC_SVD= " << AC_SVD  << endl;
  cout << "            AC_ITR= " << AC_ITR  << endl;
  cout << "            AC_HIPT=" << AC_HIPT << endl;
  cout << "            AC_OTR= " << AC_OTR  << endl;
  cout << "            AC_RICH=" << AC_RICH << endl;
  cout << "            AC_TRD= " << AC_TRD  << endl;
  cout << "            AC_ECAL=" << AC_ECAL << endl;
  cout << "            AC_MUON=" << AC_MUON << endl;
  cout << "            AC_CORR=" << AC_CORR << endl;
  cout << "            AC_GLOB=" << AC_GLOB << endl;
}

