
// *****************************************************************************
//
// source:    AC_init.C
//
// type:      source code
//
// created:   20 Oct 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    AC_init()
//
// called by: acinit_()
//
// changes:
// 03 Nov 1999 (AG) add RICH
// 19 Nov 1999 (AG) cna -> ac (alca)
// 22 Nov 1999 (AG) includes in alca
// 24 Nov 1999 (AG) improve print-out
// 12 Jan 2000 (AG) invoke gageInit();
// 01 Feb 2000 (AG) introduce flags for all subdetectors
// 27 Feb 2000 (AG) introduce alca/AC.h
// 21 Mar 2000 (AG) use modified AC_flag()
// 25 Apr 2000 (AG) use ARTE_mark()
// 18 May 2000 (AG) introduce AC_ECAL_init()
//  2 Jun 2000 (AG) use ARTE_mark()
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ONLINE includes

// Arte includes
#include "farm/farm.hh"
#include "alca/AC.h"
#include "alca/AC_SVD.hh"
#include "alca/AC_RICH.hh"
#include "alca/AC_ECAL.h"

// prototypes
int gageInit();

// externals
extern int FADEB;

// globals
int AC_SVD  = 0;
int AC_ITR  = 0;
int AC_HIPT = 0;
int AC_OTR  = 0;
int AC_RICH = 0;
int AC_TRD  = 0;
int AC_ECAL = 0;
int AC_MUON = 0;

int AC_CORR = 0;
int AC_GLOB = 0;

//
// program code
//
int
AC_init()
{
  // show up
  //cout << "AC_init()" << endl;
  ARTE_mark( "AC_init 0", 0 );

  ARTE_error( GRR_SINFO, "AC_init()", 0 );

  // print
  cout << "AC_init(): AC settings (AC kumac && comp env):" << endl;    
  cout << "           SVD=  "	<< AC_flag( COMP_SVD  ) << endl;
  cout << "           ITR=  "	<< AC_flag( COMP_ITR  ) << endl;
  cout << "           HIPT= "	<< AC_flag( COMP_HIPT ) << endl;
  cout << "           OTR=  "	<< AC_flag( COMP_OTR  ) << endl;
  cout << "           RICH= "	<< AC_flag( COMP_RICH ) << endl;
  cout << "           TRD=  "	<< AC_flag( COMP_TRD  ) << endl;
  cout << "           ECAL= "	<< AC_flag( COMP_ECAL ) << endl;
  cout << "           MUON= "	<< AC_flag( COMP_MUON ) << endl;
  cout << "           CORR= "	<< AC_flag( COMP_CORR ) << endl;
  cout << "           GLOB= "	<< AC_flag( COMP_GLOB ) << endl;
  
  // init
  int rc = 0;

  // SVD
  if( AC_flag( COMP_SVD  ) ){
    ARTE_mark( "AC_SVD_init", 0 );
    AC_SVD_init();
  }

  // RICH
  if( AC_flag( COMP_RICH ) ){
    ARTE_mark( "AC_RICH_init", 0 );
    AC_RICH_init();
  }

  // ECAL
  if( AC_flag( COMP_ECAL ) ){
    ARTE_mark( "AC_ECAL_init", 0 );
    AC_ECAL_init();
  }

  // GLOBAL
  if( AC_flag( COMP_GLOB ) ){
    ARTE_mark( "gageInit", 0 );
    rc = gageInit();
  }

  ARTE_mark( "AC_init 1", 0 );

  return 0;
}
