*-- Author :    Hartwig Albrecht   05/10/95

      SUBROUTINE ALPYTH

************************************************************************
*
* Description: generate one PYTHIA event. Option for proper treatment
*					of meson-mixing exists. 
*-- Author :    Hartwig Albrecht   05/10/95
*
* Modified by Ivan Belyaev 12/05/98 to avoid  FRITIOF hanging for
* bbar or ccbar production
*   
* Modified by J. Ivarsson and T. Lohse 04-Mar-99. 
*  - Target nucleon for Pythia randomly chosen according to target nucleus.
*  - Heavy quark jet filter based on clusters and strings containing heavy
*    quark. (subroutine ALQJET). 
*  - Selection of ONIA fragments, subroutine ALONIA
*  - Beam energy for Fritiof underlying event correctly implemented.
*  - Fritiof underlying event boosted into the Pythia lab frame.
*
* Modified by T. Lohse 16-May-99.
*  - Titanium added as valid target material (FRITIOF material NEW2)
*
* Modified by T. Lohse 10-Jun-99
*  - Lepton trigger cuts implemented
*
* Modified by T. Lohse 26-Aug-99
*  - Special pretrigger windows for electrons and muons implemented
*  - Useful in case of partially equipped detector.
*
* Called from: ALEVNT
*
* External references: LUND, PYTHIA
*
* Arguments: none
*
************************************************************************

      IMPLICIT NONE

#include "arte/ZDECAY.fi"
#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "jets/LUDAT1.fi"
#include "jets/LUDAT3.fi"
#include "jets/LUJETS.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"
#include "arte/luopts.fi"

      real evwei
      common / weight / evwei

      INTEGER NSRC, I, j,j1, j2,  MSTJ1, IFRIT, LUCOMP, imat
      INTEGER nelecs, nmuons
      CHARACTER*2 CTARG, ctup(2)
      REAL x,RLU, PLU
      REAL*8 PQQ(4),mass
      REAL betax, betay, betaz, term, xm
      REAL ptsq, thex, they

C     *
C     *     Fritiof does not like to work with initial 
C     *      beam energy less then ENERGYCUT
C     *     Let's introduce this artificial  energy cut-off.  (I.B.)
C     *
      REAL*8  ENERGYCUT , ENERGYREST 
      PARAMETER(ENERGYCUT = 5.0d0 )

      INTEGER mess_id_1, mess_id_2
      save mess_id_1, mess_id_2
      INTEGER IPT(2), NNUC(27),NPROT(27),KCD(27)
      LOGICAL flsele
      CHARACTER*4     PACD(27)
      REAL            RO1(27,2),EXMA(9,2)
      COMMON/FRCODES/IPT, PACD, NNUC, NPROT, KCD, RO1, EXMA

#include "arte/Arte.ff"

C Jetset 7.4 and Fritiof: Needs Pythia initialization for every event.
C The routines ALSAVE and ALREST work only with jetset 7.3.
C Wanted: Someone who modifies Fritiof/ALSAVE/ALREST in a way that
C the time consuming PYINIT needs to be called only at the very beginning.

      IFRIT = 1
      IF (TARGETMAT .EQ. 'P' .OR. TARGETMAT .EQ. 'N')  THEN
        IFRIT    = 0
        modefrag = 5
      END IF
      IF (MSEL.ge.4.and.modefrag.eq.5 ) IFRIT = 0

      imat = 0
      DO i=1, 27
        IF(TARGETMAT .EQ. PACD(i)) THEN
          imat = i
        ENDIF  
      ENDDO
      IF(TARGETMAT .eq. 'TI') then
        imat = 2
        NNUC(2)  = 48
        NPROT(2) = 22
        RO1(2,1) = 1.16*(1.-1.16/float(nnuc(2))**(2./3.))
        RO1(2,2) = RO1(20,2) + float(NNUC(2)-NNUC(20))*
     1             (RO1(21,2)-RO1(20,2))/float(NNUC(21)-NNUC(20))
        PACD(2) = 'TI'
      ENDIF

      IF(imat.eq.0) THEN
         call MEERRC (mess_id_1,
     +    '  ALPYTH: Target material not defined. Al is used')
         imat = 16
      ENDIF

      CTARG = 'p+'
      x = rlu(1)
      IF( float(NNUC(imat))*x.GT.float(NPROT(imat)) ) CTARG = 'n0'

      CALL PYINIT ('FIXT', 'p+', CTARG, HERAEBEAM)

C loop over generated events until requested particles found
 2    CONTINUE
      DO 20 NSRC = 1, 1000000

C PYTHIA !

C     If there is a neutral meson which mixes and should be treated 
C     differently from its anti-state, then switch of the decay
C     completely at this first stage.
C
C     This procedure can also be used for special treatment of any 
C     kind of particle, whenever duplicate special decays are to be
C     avoided.	

         IF(meson.ne.0) MDCY(LUCOMP(gKF(abs(meson))),1) = 0
         
         IF (MSEL.EQ.4.OR.MSEL.EQ.5) then
           MSTJ1 = MSTJ(1)
c  - If independent jet fragmentation is required switch off fragmentation
c    for PYEVNT
           if (modefrag.eq.3) MSTJ(1) = 0
           CALL PYEVNT
c  - filter out particles from heavy quark fragmentation
           CALL ALQJET
           MSTJ(1) = MSTJ1
         ELSEIF (MSEL.EQ.0) THEN
           CALL PYEVNT
c  - filter out particles from ONIA decays
           CALL ALONIA
         ELSE
           CALL PYEVNT
         ENDIF

C search for the special final state particle for special decay (if any)
         IF (LUNDRPART.NE.0) then
           DO  I = 1, N
             IF (K(I,1).ne.0.and.K(I,1).le.20
     +           .and.K(I,2).eq.LUNDRPART)  goto 5
           ENDDO
           goto 15
         END IF

 5       continue

C perform the decay of the selected particle (including mixing effects)
         IF(meson.ne.0) call ALSPEC

C search for the requested group of final state particles
         IF (lunsel.lt.1)  GO TO 11
         DO 10 I = 1, N
           IF (K(I,1).ne.0.and.(K(I,1).le.20.or.K(I,1).ge.31)) then
             do  j = 1, lunsel
               IF (K(I,2) .EQ. lusele(j))  GO TO 11
             enddo
           endif
 10      CONTINUE
         GOTO 15

 11      CONTINUE

C check special pretrigger requirements for electrons or muons
         IF (netrig.LE.0) GO TO 13
         nelecs = 0
         DO  I = 1, N
           IF (ABS(K(I,2)).eq.11) THEN
             IF (neacc.LE.0) THEN
               nelecs = nelecs + 1
               IF (nelecs.GE.netrig) GO TO 13
             ELSE
               ptsq = P(I,1)**2 + P(I,2)**2
               thex = P(I,3)/SQRT(P(I,1)**2+P(I,3)**2)
               thex = ACOS(thex)
               IF (P(I,1).LT.0.) thex = - thex
               they = P(I,3)/SQRT(P(I,2)**2+P(I,3)**2)
               they = ACOS(they)
               IF (P(I,2).LT.0.) they = - they
               DO  J = 1, neacc
                 IF ( ptsq.GE.ptlcut(J,1) ) THEN
                   IF ( thex.GE.txmin(J,1) .AND.
     +                  thex.LE.txmax(J,1) .AND.
     +                  they.GE.tymin(J,1) .AND.
     +                  they.LE.tymax(J,1) )  THEN
                     nelecs = nelecs + 1
                     IF (nelecs.GE.netrig) GO TO 13
                     GO TO 12
                   ENDIF
                 ENDIF
               ENDDO
 12            CONTINUE
             ENDIF
           ENDIF
         ENDDO
         GOTO 15

 13      CONTINUE

         IF (nmtrig.LE.0) GO TO 16
         nmuons = 0
         DO  I = 1, N
           IF (ABS(K(I,2)).eq.13) THEN
             IF (nmuacc.LE.0) THEN
               nmuons = nmuons + 1
               IF (nmuons.GE.nmtrig) GO TO 16
             ELSE
               ptsq = P(I,1)**2 + P(I,2)**2
               thex = P(I,3)/SQRT(P(I,1)**2+P(I,3)**2)
               thex = ACOS(thex)
               IF (P(I,1).LT.0.) thex = - thex
               they = P(I,3)/SQRT(P(I,2)**2+P(I,3)**2)
               they = ACOS(they)
               IF (P(I,2).LT.0.) they = - they
               DO  J = 1, nmuacc
                 IF ( ptsq.GE.ptlcut(J,2) ) THEN
                   IF ( thex.GE.txmin(J,2) .AND.
     +                  thex.LE.txmax(J,2) .AND.
     +                  they.GE.tymin(J,2) .AND.
     +                  they.LE.tymax(J,2) )  THEN
                     nmuons = nmuons + 1
                     IF (nmuons.GE.nmtrig) GO TO 16
                     GO TO 14
                   ENDIF
                 ENDIF
               ENDDO
 14            CONTINUE
             ENDIF
           ENDIF
         ENDDO
         GOTO 15

 16      CONTINUE

         IF(nlept.lt.1) goto 30
C check trigger conditions for leptons
         IF(nlept.eq.1) then
           DO  I = 1, N
             IF (ABS(K(I,2)).eq.11.or.ABS(K(I,2)).eq.13) THEN
               IF (P(I,1)**2+P(I,2)**2.GT.ptl1) GO TO 30
             ENDIF 
           ENDDO
         ELSEIF(nlept.eq.2) then
           DO  I = 1, N
             IF (ABS(K(I,2)).eq.11.or.ABS(K(I,2)).eq.13) THEN
               IF (P(I,1)**2+P(I,2)**2.GT.ptl1) THEN
                 DO  j = 1, N
                   IF (j.ne.I .and.
     +                 (ABS(K(j,2)).eq.11.or.ABS(K(j,2)).eq.13)) THEN
                     IF (P(j,1)**2+P(j,2)**2.GT.ptl2) THEN
                       IF (xml12.LE.0.)         GO TO 30
                       xm =   (P(I,4) + P(j,4))**2
     +                       -(P(I,1) + P(j,1))**2
     +                       -(P(I,2) + P(j,2))**2
     +                       -(P(I,3) + P(j,3))**2
                       IF (xm.gt.xml12)         GO TO 30
                     ENDIF
                   ENDIF
                 ENDDO
               ENDIF
             ENDIF
           ENDDO
         ELSE
           goto 30
         ENDIF

 15      CONTINUE
         IF (NSRC .EQ. 20)  CALL MEMESC (mess_id_2, 10, ILARGEST,
     +     'ALPYTH: more than 20 searches for the requested particles')

 20   CONTINUE

      CALL MEERRO (
     +    'ALPYTH: infinit loop in search for the requested particles')
      GO TO 2

 30   continue

C working space for pointers from LUJETS to MTRA; pointer < 0: discarded

      CALL MMBWRK (JTEMP, N, 2, 0)

C search for the primary quarks and its kin

      PQQ(1) = 0.d0
      PQQ(2) = 0.d0
      PQQ(3) = 0.d0
      PQQ(4) = 0.d0

      DO 40 I = 1, N

c     remove empty and comment lines:

         IF (K(I,1) .EQ. 0)  THEN
            IQ(L_A(1,I,JTEMP)) = -1
         ELSE IF (K(I,1) .GE. 21 .AND. K(I,1) .LE. 30)  THEN
            IQ(L_A(1,I,JTEMP)) = -1
c resonance from onium production
         ELSE IF (MSEL.eq.0) THEN
            j1 = MOD(ABS(K(I,2)),1000)/100
            j2 = MOD(ABS(K(I,2)),10000)/1000
            if (j1.eq.4.or.j1.eq.5.or.j2.eq.4.or.j2.eq.5)
     +         IQ(L_A(1,I,JTEMP)) = 1
C cluster/string/independent-jet from heavy quarks
         ELSE IF (ABS(K(K(I,3),2)) .EQ. MSEL)  THEN
            IQ(L_A(1,I,JTEMP)) = 1
         END IF

         IF ( K(I,4) .EQ. 0)  THEN
C     add 4-momentum of all stable particles                 
            PQQ(1) = PQQ(1) + dble(P(I,1))
            PQQ(2) = PQQ(2) + dble(P(I,2))
            PQQ(3) = PQQ(3) + dble(P(I,3))
            PQQ(4) = PQQ(4) + dble(P(I,4))
         END IF

 40   CONTINUE

C fill Arte tables

      CALL ALMCIN
      CALL ALMTRA

C the rest of the event is now generated by Fritiof.
C Fritiof is not yet able to use the full 4-vector PQQ.
C Note that Fritiof is initialized every time.
C The initialization FRPYINI is called up to 4 times / event.
C Someone has to streamline this program.


      IF (IFRIT .EQ. 0)  GO TO 90

      CALL ALSAVE
      MSEL = 1
C     *
C     * Before invoking FRITIOF, check the fritiof energy.
C     * in b-bar or ccbar production, fritiof likes to hang up
C     * if the total energy less then ENERGYCUT
C     * Just skip FRITIOF for these events.
C     * Introduce a small energy nonconservation of about ENERGYCUT for 
C     * these quite rare events.
C     *                                  (I.B.)
C     * This small energy can be regarded as part of the energy loss 
C     * by the nuclear remnant which is anyway not treated by Fritiof 
C     * correctly.
C     *                                  (J.I. and T.L.)


      mass = 0.939d0*dble(NNUC(imat))

      ENERGYREST = dble(HERAEBEAM) - PQQ(4)  -  0.5d0/mass*(  
     +         (PQQ(4)-PQQ(3)) * (2.d0*dble(HERAEBEAM)-PQQ(4)-PQQ(3)) +
     +                         PQQ(1)**2+PQQ(2)**2 )

      IF( ENERGYREST .LE. ENERGYCUT ) THEN 
c
         C78='Alpyth.F warning! FRITIOF Energy='
         CALL CH78AF( sngl(ENERGYREST) , -3 )
         CALL CH78AC(' < ' ) 
         CALL CH78AF( sngl(ENERGYCUT) , -3 ) 
         CALL ME78PC( 9237 , 300 ) 
c     * Empty LUND event record 
         N = 0
      ELSE 

         CALL FREVENT ('FIXT', 'P', PACD(imat), sngl(ENERGYREST) )

c boost Fritiof event into CMS

         betax = 0.
         betay = 0.
         betaz = -sngl(energyrest/(mass+energyrest))
         call lurobo(0.,0.,betax, betay, betaz)

c boost CMS into lab system
         term = 1./sngl(heraebeam + mass - PQQ(4))
         betax = -PQQ(1)*term
         betay = -PQQ(2)*term
         betaz = (heraebeam-PQQ(3))*term
         call lurobo(0.,0.,betax, betay, betaz)

      ENDIF
 
      CALL ALREST
      CALL ALFSTO

 90   END

************************************************************************
* End of ALPYTH
************************************************************************


