//
// This file is part of the C++ interface for Arte
// the HERA-B reconstruction and analysis tool
//
// This is the definition of the ArtePointer class.
//
// ArtePointer allows access to the Arte data tables in
// a pointer-like way.
//
//
// .SS Usage // A Pointer to an GESL is declared as
//
// .ft B
//      ArteRelationPointer<xxx> relPoint;
//
//
// .SS See Also
//
// ArtePointer.hh
//
// Author: Christoph Borgmeier
//

#ifndef _ARTE_RELATION_POINTER_HH_
#define _ARTE_RELATION_POINTER_HH_

//#ifdef __GNUC__
//#pragma interface
//#endif

#include "arte/Pointer.hh"

template <class t>
struct ArteOneToManyRelation
{
  ArtePointer<t> el;
  ArtePointer<t> next;
};


template <class t>
struct ArteManyToManyRelation
{
  ArtePointer<t> el1;
  ArtePointer<t> next2;
  ArtePointer<t> el2;
  ArtePointer<t> next1;
};


template <class t>
struct ArteRelation;


template <class t>
class ArteRelationPointer
{
public:
  ArteRelationPointer();
  ArteRelationPointer( const ArteRelationPointer<t>& arg );

  // inline t& operator*() const;
  // Dereferences the pointer.

  // inline t* operator->() const;
  // Dereferences the pointer.

  inline static int relType();

protected:
  inline const ArtePointer<t> pointer() const;

  inline ArtePointer<t> & getBase();
  inline const ArtePointer<t> getBase() const;

  inline ArteManyToManyRelation<t>& getManyToMany() const;
  inline ArteOneToManyRelation<t>& getOneToMany() const;

private:
  ArtePointer<t> base;
};

#include "arte/RelationPointer.icc"

#endif
