//                              -*- Mode: C++ -*-
//: EAClusterAnalyzer.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 4/4/97
// Last Update ........
//

#ifndef EACLUSTERANALYZER_HH_
#define EACLUSTERANALYZER_HH_

#include "care/ECALtypes.hh"
class  EMClusterMember;
class  EMHitCell;
class  EMCellFraction;
class  EMCluster;
class  EMTrackOnEcal;
class  EMClusterTree;

/**
class EAClusterAnalyzer.
  Abstract base class for cluster analysis.
  see EMClusterMember, EMClusterMember, EMHitCell, EMTrackOnEcal, 
      EMClusterTree
*/


class EAClusterAnalyzer {
public:
  //-- Constructors
  /// Only default constructor is provided
  EAClusterAnalyzer();
  /// The virtual default destructor
  virtual ~EAClusterAnalyzer();

  /**@name Beheavioral methods */
  //@{ 
  /// EMClusterMember Analyzer
  virtual void doAnalysis(const EMClusterMember * clsMem) = 0;
  //--
  /// EMHitCell Analyzer
  virtual void doAnalysis(const EMHitCell * hitT) = 0;
  //--
  /// EMCellFraction Analyzer
  virtual void doAnalysis(const EMCellFraction * hitT);

  //--
  /// EMCluster Analyzer (no loop on childs)
  virtual void doAnalysis(const EMCluster * clus);
  //--
  /// EMCluster Analyzer (no loop on childs)
  virtual void doAnalysis(const EMClusterTree * tree);
  //--
  /// EMTrackOnEcal Analyzer
  virtual void doAnalysis(const EMTrackOnEcal * track);
  //@}
private:
};


#endif // EACLUSTERANALYZER_HH_
