//                              -*- Mode: C++ -*-
//:ELNearCells.hh
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/5/97
/*
 $Log: ELNearCells.hh,v $
 Revision 1.1  1999/08/04 15:21:57  natasha
 new care version from Mauro

# Revision 1.1  1999/06/01  16:09:00  villa
# before HH
#
 */

#ifndef ELNEARCELLS_HH_
#define ELNEARCELLS_HH_

#include "care/ECALtypes.hh"
/**
class  ELNearCells.
  Get the list of nearby cells and provide access to the list.
  Cells may or may not be clusterized.
*/

class ELNearCells {
public:
  /// Enumeration to get the near relation type
  enum NearRel {crux, square};
  //--
  ///  constructor
  inline ELNearCells();
  //--
  ///  destructor
  inline ~ELNearCells();

  /**@name Beheavioral methods */
  //@{ 
  //--
  ///  Get neighbouring cells of a given cell
  inline void of(CellIdx_t cell, NearRel rel = crux);
  //@}
 
  /**@name Iterator methods */
  //@{ 
  //--
  //  start an iteration 
  inline void begin();
  //--
  //  start an iteration 
  inline void begin(CellIdx_t cellidx, NearRel rel = crux);
  //--
  //  access member
  inline bool more() const;
  //--
  //  consider next cell
  inline void next();
  //--
  //  get the current cell
  inline CellIdx_t operator*() const;
  //--
  ///  get the number of neighbours
  inline unsigned int  howMany() const;
  //--
  ///  Test if the cell is in the neighbourhood of the center
  inline bool isNear(CellIdx_t cellidx) const;
  //@}

protected:
  //--
  /// Center cell
  CellIdx_t cell;
  ///  Number of stored neighbours
  unsigned int numNeib;
  ///  number of accessed neighbours
  unsigned int numacc;
};

// ----------- definitions of inline functions

ELNearCells::ELNearCells() : cell(0), numNeib(0), numacc(0) {}
ELNearCells::~ELNearCells(){};

//  Get neighbouring cells of a given cell
void ELNearCells::of(CellIdx_t cellidx, ELNearCells::NearRel rel) {
  cell = cellidx;
  numacc = 0;
  numNeib = (cell->neib & 0xFF);
  if( rel==ELNearCells::square ){
    numNeib += (cell->neib & 0xFF00)>>8;
  }
}

//--
//  start an iteration 
void ELNearCells::begin(){
  numacc = 0;
}

  //  start an iteration 
void ELNearCells::begin(CellIdx_t cell, ELNearCells::NearRel rel){
  of(cell, rel);
  begin();
}

//  access member
bool ELNearCells::more() const{
  return (numacc<numNeib);
}

//  consider next cell
void ELNearCells::next(){
  numacc++;
}

//  get the current cell
CellIdx_t ELNearCells::operator*() const{
  ArtePointer<GNEI> nei(cell->adrs + numacc);
  return CellIdx_t(nei->ncod);
}

//  get the number of neighbours
unsigned int  ELNearCells::howMany() const{
  return numNeib;
}

//  Test if the cell is in the neighbourhood of the center
bool ELNearCells::isNear(CellIdx_t cellidx) const{
  int gcce = cellidx.index();
  ArtePointer<GNEI> nei(cell->adrs);
  unsigned int i=0;
  for(; i<numNeib && nei->ncod!=gcce; ++i, ++nei){}
  return (i<numNeib); // almost like nei->ncod==gcce but safer!
}

#endif // ELNEARCELLS_HH_
