//functions for matrix calculations
//the matrix has to be stored in a vector<double>
//in the form:
//( a11 a12 ... a1n)
//( a21 a22 ... a2n)
//    .........       ={a11,a21,...,am1,a12,a22,....,amn}
//    .........
//( am1 a12 ... amn)
// which means fortran format

#ifndef ROVERMATRIX_H
#define ROVERMATRIX_H
#include "rovermatrix.h"
#endif
 

//matrixinversion with the CERN-routine,the resulting vector overwrites a

int mtxinv(vector<double>& a, int n)
{
  int   ifail;
  double *help = new double[n*n];
  double *workspace = new double[n];

  for(int i=0;i<n*n;++i)
     {
      help[i] = a[i];
     }    
 
  dinv_(&n, help, &n,workspace, &ifail);

 
  for(int i=0;i<n*n;++i)
     {
      a[i] = help[i];
     } 

  delete[] help;
  delete[] workspace;
  
  return ifail;
}  

//  invert a square matrix and print out informations 

int mtxinv(vector<double>& a, int n,bool check)
{

  int   ifail;
  double *help = new double[n*n];
  double *workspace = new double[n];
  for(int i=0;i<n*n;++i)
     {
      help[i] = a[i];
     }    

  cout << "matrix to invert" << endl;   
  for(int i=0;i<n;++i)
     {
      for(int j=0;j<n;++j)
         {
	  cout << help[i+n*j] << "\t";
	 }
      cout << endl;	  
     } 

 
  dinv_(&n, help, &n,workspace, &ifail);

  cout << "inverted matrix" << endl;   
  for(int i=0;i<n;++i)
     {
      for(int j=0;j<n;++j)
         {
	  cout << help[i+n*j] << "\t";
	 }
      cout << endl;	  
     } 


  double *test = new double[n*n];
  double fom = 0.;
  cout << "test of matrixinversion" << endl;   

  int ka,kb,kr;
  for(int i=1;i<=n;++i)
     {
       for(int j=1;j<=n;++j)
          {
            kr=(j-1)*n+(i-1);
            test[kr]=0.;
             
            for(int k=1;k<=n;++k)
               {
                ka=(k-1)*n+(i-1);
                kb=(j-1)*n+(k-1);
                test[kr] += a[ka]*help[kb];
               }
	     cout << test[kr] << "\t";
             fom += fabs(test[kr]);
            }
	cout << endl;   
       }

  fom /= double(n);
  cout << "fom: " << fom << endl;  
  for(int i=0;i<n*n;++i)
     {
      a[i] = help[i];
     } 

  delete[] test;   
  delete[] help;
  delete[] workspace;
  
  return ifail;
}  

// multiply (n/l) matrix a with (l/m) matrix b,result (n/m) r
  
int mtxmul(vector<double> a,vector<double> b,vector<double>& r,
                    int n,int l,int m)
{
  int i,k,j,ka,kb,kr;
  for(i=1;i<=n;++i)
     {
       for(j=1;j<=l;++j)
          {
            kr=(j-1)*n+(i-1);
            r[kr]=0;
             
            for(k=1;k<=l;++k)
               {
                ka=(k-1)*n+(i-1);
                kb=(j-1)*l+(k-1);
                r[kr]=r[kr]+a[ka]*b[kb];
               }
            }
       }

   return 0;
}

// multiply r with scalar
int mtxscal(double scal,vector<double>& r)
{
 for(vector<double>::iterator iter = r.begin();iter != r.end();++iter)
    {
     *iter = scal * (*iter);
    }
     
 return 0;

}

int mtxadd(vector<double>& a,vector<double>& b,vector<double>& c)
{
  if(a.size() != b.size() || a.size() != c.size() )
    {
     return -1;
    }

 for(int i=0;i != int(c.size());++i)
    {
     c[i] = a[i] + b[i];
    }  
 return 0;
}     
// singular value decomposition, returns condition number of a
// singular values are stored in sv
double mtxsvdcmp(vector<double>& a,int n,vector<double>& sv)
{
 vector< vector<double> > av;
 vector< vector<double> > an;
 vector<double> sval(n+1,0.);
 av.resize(n+1);
 an.resize(n+1);
 for(int i=1;i<int(av.size());++i)
    {
     av[i].resize(n+1);
     an[i].resize(n+1);
     for(int j=1;j<int(av[i].size());++j)
        {
	 av[i][j] = a[(i-1)+(j-1)*n];
	 an[i][j] = 0.;
	}
     }	
       
 svdcmp_nrc(av,n,n,sval,an);
 sv.clear();
 double max = sval[1];
 double min = sval[1];
 
 for(int i=1;i<=n;++i)
    {
     if(sval[i] > max)
       {
        max = sval[i];
       }	
     if(sval[i] < min)
       {
        min = sval[i];
       }	
     sv.push_back(sval[i]);
    }
    
 return (max/min);
} 
  
//solve equation Ax=b, result x is stored in b
int lusolve(vector<double> a,int n,vector<double>& b)
{ 

 vector<int> index(n+1,0.);  
 
 double d = 0.;

 if(ludcmp_nrc(a,n,index,d))
   {
    return -1;
   } 
 lubksb_nrc(a,n,index,b);

 return 0;
}


int luinverse(vector<double>& a,int n)
{
 vector<int> index(n+1,0.);  
 vector<double> result(n,0.);
 vector<double> ainv(n*n,0.);

 double d = 0.;

 if(ludcmp_nrc(a,n,index,d))
   {
    return -1;
   } 

 for(int j=0;j<n;++j)
    {
     for(int k=0;k<n;++k)
        {
	  result[k] = 0.;
        }
     result[j] = 1.0;
     lubksb_nrc(a,n,index,result); 
    for(int i=0;i<n;++i)
       {
        ainv[j+i*n] = result[i];
       }
    }
     
  for(int k=0;k<n*n;++k)
     { 
      a[k]=ainv[k];
     }
   return 0;   
}
    


void svdcmp_nrc(vector< vector<double> >& a,int m,int n,vector<double>& w, vector< vector<double> >& v)
{
	int flag,i,its,j,jj,k,l=0,nm=0;
	double c,f,h,s,x,y,z;
	double anorm=0.0,g=0.0,scale=0.0;
	double *rv1;
    
	if (m < n)
	   {
	    nrerror("SVDCMP: You must augment A with extra zero rows");
	    return;
	   } 
	rv1= new double[n];
	for (i=1;i<=n;i++) {
		l=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;
		if (i <= m) {
			for (k=i;k<=m;k++) scale += fabs(a[k][i]);
			if (scale) {
				for (k=i;k<=m;k++) {
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f=a[i][i];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][i]=f-g;
				if (i != n) {
					for (j=l;j<=n;j++) {
						for (s=0.0,k=i;k<=m;k++) s += a[k][i]*a[k][j];
						f=s/h;
						for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
					}
				}
				for (k=i;k<=m;k++) a[k][i] *= scale;
			}
		}
		w[i]=scale*g;
		g=s=scale=0.0;
		if (i <= m && i != n) {
			for (k=l;k<=n;k++) scale += fabs(a[i][k]);
			if (scale) {
				for (k=l;k<=n;k++) {
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][l];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][l]=f-g;
				for (k=l;k<=n;k++) rv1[k]=a[i][k]/h;
				if (i != m) {
					for (j=l;j<=m;j++) {
						for (s=0.0,k=l;k<=n;k++) s += a[j][k]*a[i][k];
						for (k=l;k<=n;k++) a[j][k] += s*rv1[k];
					}
				}
				for (k=l;k<=n;k++) a[i][k] *= scale;
			}
		}
		anorm=MAX(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
	for (i=n;i>=1;i--) {
		if (i < n) {
			if (g) {
				for (j=l;j<=n;j++)
					v[j][i]=(a[i][j]/a[i][l])/g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[i][k]*v[k][j];
					for (k=l;k<=n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=l;j<=n;j++) v[i][j]=v[j][i]=0.0;
		}
		v[i][i]=1.0;
		g=rv1[i];
		l=i;
	}
	for (i=n;i>=1;i--) {
		l=i+1;
		g=w[i];
		if (i < n)
			for (j=l;j<=n;j++) a[i][j]=0.0;
		if (g) {
			g=1.0/g;
			if (i != n) {
				for (j=l;j<=n;j++) {
					for (s=0.0,k=l;k<=m;k++) s += a[k][i]*a[k][j];
					f=(s/a[i][i])*g;
					for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
				}
			}
			for (j=i;j<=m;j++) a[j][i] *= g;
		} else {
			for (j=i;j<=m;j++) a[j][i]=0.0;
		}
		++a[i][i];
	}
	for (k=n;k>=1;k--) {
		for (its=1;its<=30;its++) {
			flag=1;
			for (l=k;l>=1;l--) {
				nm=l-1;
				if (fabs(rv1[l])+anorm == anorm) {
					flag=0;
					break;
				}
				if (fabs(w[nm])+anorm == anorm) break;
			}
			if (flag) {
				c=0.0;
				s=1.0;
				for (i=l;i<=k;i++) {
					f=s*rv1[i];
					if (fabs(f)+anorm != anorm) {
						g=w[i];
						h=PYTHAG(f,g);
						w[i]=h;
						h=1.0/h;
						c=g*h;
						s=(-f*h);
						for (j=1;j<=m;j++) {
							y=a[j][nm];
							z=a[j][i];
							a[j][nm]=y*c+z*s;
							a[j][i]=z*c-y*s;
						}
					}
				}
			}
			z=w[k];
			if (l == k) {
				if (z < 0.0) {
					w[k] = -z;
					for (j=1;j<=n;j++) v[j][k]=(-v[j][k]);
				}
				break;
			}
			if (its == 30) nrerror("No convergence in 30 SVDCMP iterations");
			x=w[l];
			nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=PYTHAG(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0;
			for (j=l;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=PYTHAG(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g=g*c-x*s;
				h=y*s;
				y=y*c;
				for (jj=1;jj<=n;jj++) {
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=PYTHAG(f,h);
				w[j]=z;
				if (z) {
					z=1.0/z;
					c=f*z;
					s=h*z;
				}
				f=(c*g)+(s*y);
				x=(c*y)-(s*g);
				for (jj=1;jj<=m;jj++) {
					y=a[jj][j];
					z=a[jj][i];
					a[jj][j]=y*c+z*s;
					a[jj][i]=z*c-y*s;
				}
			}
			rv1[l]=0.0;
			rv1[k]=f;
			w[k]=x;
		}
	}
	
	delete[] rv1;
}

int ludcmp_nrc(vector<double>& a,int n,vector<int>& indx,double& d)
{
	int i,imax=0,j,k;
	double big,dum,sum,temp;
	double *vv;
        double TINY = 1.0e-20;
	vv = new double[n+1]; 
	d=1.0;
	for (i=1;i<=n;i++) {
		big=0.0;
		for (j=1;j<=n;j++)
			if ((temp=fabs(a[(i-1)+(j-1)*n])) > big) big=temp;
		if (big == 0.0){ cerr << "Singular matrix in routine LUDCMP" << endl;
                  	         delete[] vv;
		                 return -1;}
		vv[i]=1.0/big;
	}
	for (j=1;j<=n;j++) {
		for (i=1;i<j;i++) {
			sum=a[(i-1)+(j-1)*n];
			for (k=1;k<i;k++) sum -= a[(i-1)+(k-1)*n]*a[(k-1)+(j-1)*n];
			a[(i-1)+(j-1)*n]=sum;
		}
		big=0.0;
		for (i=j;i<=n;i++) {
			sum=a[(i-1)+(j-1)*n];
			for (k=1;k<j;k++)
				sum -= a[(i-1)+(k-1)*n]*a[(k-1)+(j-1)*n];
			a[(i-1)+(j-1)*n]=sum;
			if ( (dum=vv[i]*fabs(sum)) >= big) {
				big=dum;
				imax=i;
			}
		}
		if (j != imax) {
			for (k=1;k<=n;k++) {
				dum=a[(imax-1)+(k-1)*n];
				a[(imax-1)+(k-1)*n]=a[(j-1)+(k-1)*n];
				a[(j-1)+(k-1)*n]=dum;
			}
			d = -d;
			vv[imax]=vv[j];
		}
		indx[j]=imax;
		if (a[(j-1)+(j-1)*n] == 0.0) a[(j-1)+(j-1)*n]=TINY;
		if (j != n) {
			dum=1.0/(a[(j-1)+(j-1)*n]);
			for (i=j+1;i<=n;i++) a[(i-1)+(j-1)*n] *= dum;
		}
	}

 return 0;	
}

void lubksb_nrc(vector<double>& a,int n,vector<int>& indx,vector<double>& b)
{
	int i,ii=0,ip,j;
	float sum;
    
	for (i=1;i<=n;i++) {
		ip=indx[i];
		sum=b[ip-1];
		b[ip-1]=b[i-1];
		if (ii)
			for (j=ii;j<=i-1;j++) sum -= a[(i-1)+(j-1)*n]*b[j-1];
		else if (sum) ii=i;
		b[i-1]=sum;
	}
	for (i=n;i>=1;i--) {
		sum=b[i-1];
		for (j=i+1;j<=n;j++) sum -= a[(i-1)+(j-1)*n]*b[j-1];
		b[i-1]=sum/a[(i-1)+(i-1)*n];
	}
}



double PYTHAG(double a,double b)
{
 double ct = 0.;
 
 if(fabs(a) > fabs(b))
   {
    ct = b/a;
    return double(a*sqrt(1.+ct*ct));
   }
 else
   {
    ct = a/b;
   return double(b*sqrt(1.+ct*ct));
   }    

} 

double MAX(double a,double b)
{
 if(a > b)
   {
    return a;
   }
 else
   {
    return b;
   }
} 

double SIGN(double a,double b)
{
 if(b>= 0.0)
   {
    return double(fabs(a));
   }
 else
   {
    return double(-fabs(a));
   }
}                 

void nrerror(char *error_text)
{
    
	cerr << "SVD run-time error..." << endl;
	cerr << error_text << endl;
}


