#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


extern DbFileDesc *DbFileDescList;

void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
int  mmrstat_(int *, int *);
int  mmwstat_(int *, int *);
DbTableDesc *DbArteTableDesc(char *);
int DbMatchType(char *opt);


void DbWriteHeader(int lun, int *Mlink, int major, int minor, int *ierr)
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Write to a MIZZI file a header bank and its subs.
   *
   * Called from: anywhere.
   *
   * External references: dbwtab_
   *
   * Argument in/out description
   *  lun     in     LUN
   *  Mlink   in     header bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  ierr    out    operation status
   *              0  OK
   *              1  Mlink not inside link area
   *              2  Bank name too long for MIZZI
   *              3  Bank is not the header one
   *              4  Bank and all subs are empty
   *              5  Bank is not marked for output
   *           >100  100*nlink+(err status from dbwtab_ routine)
   *                 dbwtab reported error while writing bank # nlink
   *
   *
   ************************************************************************/

  int n, nlink, *Link, sub, retval=1;
  char mess[80]="DBWRIT: ", warn[80]="DBWRIT: ";
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  int var;
  
  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;


  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
	*ierr=1;
    return;
  }

  var = DB_MAX_NAME_SIZE/4;
  chhtot_( MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen > %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
	*ierr=2;
    return;
  }
  
  /* Check if table is a header bank. */

  if( MEMO(nlink).sup[0] != 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not a header one",Name);
    mecErrorCount( &errPtr, mess );
	*ierr=3;
    return;
  }

  /* 
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* 
   * Check if the bank is marked for output. 
   */

	
	var=mmwstat_( &lun, &nlink );
  		
  	if(var>1)	
		{
		static int errPtr;
		sprintf(mess+strlen(mess),
		"header bank name \"%s\" is not marked for output", Name);
		mecErrorCount( &errPtr, mess );
		*ierr=5;
		return;
		}
		
	/* Check if there are non-empty banks */
		
	if(var==1)
	{
		static int errPtr;
		int i=0;
		for(n=MEMO(nlink).sub1;n <= MEMO(nlink).sub2; n++)
		{
			int sub;
			sub=MEMS(n).sub;
			if(mmwstat_(&lun, &sub)==0)
				i++;
		}
		if(i==0)
		{ 
			sprintf(mess+strlen(mess),
				"header bank \"%s\" and all subs are empty", Name);
			mecErrorCount( &errPtr, mess );
			*ierr=4;
			return;
		}
  	}


  /* Write out header bank ... */

  sprintf(warn,
     "DBWRIT: writing header table %s on %s", Name , Csym );
     {
     static int errPtr;
     mecWarningCount( &errPtr, warn );
     }

  DbWriteTable( lun, Mlink, major, minor, &retval );
  if(retval)
  {
	  *ierr=retval+nlink*100;
	  return;
  }

  /* ... and all subs. */
  

  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) {
    int res;
    
    sub = MEMS(n).sub;

    if( sub == nlink ) 
	    continue;

    res = mmwstat_( &lun, &sub );
    
    if(res == 2 )
            continue;
    if( res > 1 ) 
	  {
      static int errPtr;
      var = DB_MAX_NAME_SIZE/4;
      chhtot_(MEMO( sub ).name, Name, &var );
      sprintf(mess+8,
         "subbank %s is %s, skipped", Name, 
         	res==1?"empy":"not marked for output");
      mecWarningCount( &errPtr, mess );
      continue;
      }

 
  var = DB_MAX_NAME_SIZE/4;
  chhtot_( MEMO(sub).name, Name, &var );

   sprintf(warn,
     "DBWRIT: writing table %s", Name );
     {
     static int errPtr;
     mecWarningCount( &errPtr, warn );
     }
   
    DbWriteTable( lun, (int *)((artelink_.larln1 + sub ) * 4),
    		 major, minor, &retval );
	if(retval)
	{
		*ierr=retval+sub*100;
		return;
	}
    
  }
  
  /* Save tables holding many-many relations */
  if(!DbLedaCompatibility)
  {
  int mema, nCol, i;
  mema=MEMO(nlink).mema;
  nCol=MEMO(nlink).ocol;
  for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;

  		DbWriteTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 		major, minor, &retval );
  		}
  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) 
    {
    sub = MEMS(n).sub;
    if( sub == nlink ) 
	    continue;
    mema=MEMO(sub).mema;
    nCol=MEMO(sub).ocol;
    for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;
 
  		DbWriteTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 		major, minor, &retval );
  		}
     }
     
  }
  
  *ierr=0;
  return;

}
