*
*  B.Fominykh  18.11.96
*  calculates the stop energy for electron entered to TUBE or PAD muon
*  chamber taking into account dE/dX and soft bremstrahlung
*  routine DGMUCUT is called from DGMUOO
*  coordinate system - muon chamber system (x>0., y>0. inside chamber)
*  input parameters: MS   - muon detector superlayer (= 1 - 4)
*                    IPL  - chamber number
*                    IW   - hitted wire number (= 1 - 16)
*                    P    - momentum im i.p.
*                    XIP  - x-coordinate of impact point to detector
*                    YIP  - y-coordinate of impact point to detector
*                    ZIP  - z-coordinate of impact point to detector
*                    TX   - Px/Pz in impact point to detector
*                    TY   - Py/Pz in impact point to detector
*  output parameter: EST  - stop energy for electron in GeV
*
      SUBROUTINE DGMUCUT(MS,IPL,IW,P,XIP,YIP,ZIP,TX,TY,EST)
C     ====================================================
*
      IMPLICIT NONE
      INTEGER MS,IPL,IW,NW,II,J
      REAL P,XIP,YIP,ZIP,TX,TY,EST
      REAL ZD1,ZD2,DW,AC,XFW,XD,YD
      REAL WW,CSI,SNI,DD
* table of range as function of energy for electrons in Al
      REAL ET(31),RAT(31),RHOAL
#include "arte/Arte.fi"
#include "auto/GEO.fi"

      DATA RHOAL /2.70/
      DATA RAT /0., 3.519E-4,1.864E-2,2.243E-1,5.493E-1,8.825E-1,
     +          1.212, 1.601, 1.855, 2.168, 2.476, 3.076, 3.658,
     +          4.225, 4.777, 5.315, 5.841, 10.54, 14.48, 17.88,
     +          20.87, 23.55, 28.16, 32.04, 45.55, 54.23, 60.62,
     +          65.69, 69.88, 76.58, 81.84 /
      DATA ET  /0., 0.00001, 0.0001, 0.0005, 0.001, 0.0015, 0.002,
     +              0.0026 , 0.003 , 0.0035, 0.004, 0.005 , 0.006,
     +              0.007  , 0.008 , 0.009 , 0.01 , 0.02  , 0.03 ,
     +              0.04   , 0.05  , 0.06  , 0.08 , 0.1   , 0.2  ,
     +              0.3    , 0.4   , 0.5   , 0.6  , 0.8   , 1. /


#include "arte/Arte.ff"
#include "auto/GEO.ff"
C----+===1=========2=========3=========4=========5=========6=========7==*******
* debug print of input info
*      WRITE(LLOGF,1001) MS,IPL,IW,P,XIP,YIP,ZIP,TX,TY
* 1001 FORMAT(X,'MS,IPL,IW',I2,I6,I3,'  P',F8.3,
*     +        '  XIP,YIP,ZIP',3F8.2,'  TX,TY',2F8.2)

* input check-point
      EST=0.
      IF(MS.LT.1.OR.MS.GT.4) GOTO 999
      IF(IPL.LT.1)           GOTO 999
* number of wires in detector
      NW=IGEDE_NWI(1,IPL)
      IF(IW.LT.1.OR.IW.GT.NW) GOTO 999
* z-position of begining and end of MC-volume (detector)
      ZD1=FGEDE_ZMIN(IPL)
      ZD2=FGEDE_ZMAX(IPL)
      IF(ZIP.LT.ZD1.OR.ZIP.GT.ZD2) GOTO 999
* get parameters of detector IPL in detector coordinate system
* definition: min-X min-Y corner has coordinates x=0,y=0
      DW   =  FGEDE_PIT(IPL)   ! distance between wires
      AC   =  FGEDE_DOD(IPL)   ! half of x-size of sensetive volume
      XFW  =  FGEDE_XFW(1,IPL) ! X-position of first wire
      XD   =  FGEDE_XMAX(IPL)-FGEDE_XMIN(IPL)    ! x-size of detector
      YD   =  FGEDE_YMAX(IPL)-FGEDE_YMIN(IPL)    ! y-size of detector

* debug print of detector parameters
*      WRITE(LLOGF,1002) NW,ZD1,ZD2,XD,YD,DW,XFW,AC
* 1002 FORMAT(X,'NW,ZD1,ZD2',I3,2F8.2,'  XD,YD',F6.3,F8.2
*     +         '  DW,XFW,AC',3F6.3)

* calculation of Al thickness befor sensitive volume
* angles
      WW=TX*TX+TY*TY
      CSI=SQRT(1.+WW)
      IF(WW.GT.1.E-6) THEN
        SNI=SQRT(1.+1./WW)
      ELSE
        SNI=1.E+3
      ENDIF
* thickness
      DD=0.
* front and rare corver
      IF(ABS(ZIP-ZD1).LT.0.05.OR.ABS(ZIP-ZD2).LT.0.05) DD=DD+CSI
* side corver
      IF(ABS(XIP).LT.0.05.OR.ABS(XIP-XD).LT.0.05.OR.
     +   ABS(YIP).LT.0.05.OR.ABS(YIP-YD).LT.0.05) DD=DD+SNI
* inner partitions
      II=INT((XIP-XFW)/DW+0.5)+1
      IF(II.LT.1) II=1
      IF(II.GT.NW) II=NW 
      DD=DD+ABS(REAL(II-IW))*SNI
      DD=DD*(0.5*DW-AC)*RHOAL
* transform range to stopping energy
      DO 20 J=2,31
        IF(DD.LT.RAT(J)) GOTO 21
  20  CONTINUE
      GOTO 999
  21  EST=ET(J-1)+(DD-RAT(J-1))*(ET(J)-ET(J-1))/(RAT(J)-RAT(J-1))

 999  CONTINUE

* debug print of results
*      WRITE(LLOGF,1003) CSI,SNI,II,DD,EST
* 1003 FORMAT(X,'CSI,SNI',2F7.2,'  II,DD,EST',I3,F9.3,F9.5)
*      WRITE(LLOGF,*) '  '
      RETURN
      END
