
// ******************************************************************
//
// source:    DQ_FARM_hit_evnt.C
//
// type:      source code
//
// created:   10 Aug 1999
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ_evnt()
//
// changes:
// 07 Sep 1999 (AG) fill processing time histo
// 09 Sep 1999 (AG) polishing
// 21 Oct 1999 (AG) polishing
// 27 Oct 1999 (AG) use defines from DQ.h
// 29 Oct 1999 (AG) redefine histogram IDs
//  1 Nov 1999 (AG) get rid of dq_hid.h
// 17 May 2000 (AG) histogram some raw table sizes
// 24 May 2000 (AG) histogram table size in Byte
// 26 May 2000 (AG) histogram time_rcevnt here
// 31 May 2000 (AG) change scale for DSVD, DITR, DOTR
// 23 Jun 2000 (AG) add hit occupancy histos
// 26 Jun 2000 (AG) added ARTE_mark()
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_FARM.h"

// externals
extern float time_rcevnt;

//
// function code
//
int
DQ_FARM_hit_evnt()
{
  // show up
  ARTE_mark( "DQ_FARM_hit_evnt 0", 0 );

  //====================================================================
  //       Trigger Code
  //====================================================================
  
  float FCS_tcode = faDDAQFCS_tcode();
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 1 ), FCS_tcode, 1 );
  
  //====================================================================
  //       Target Rate
  //====================================================================
  
  float ia = faDTARia();
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 2 ), ia, 1 ); // [MHz]
  
  //====================================================================
  //   get information about slp/farm-nodes
  //====================================================================
  
  int slp_node = faDDAQslp_node();
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 3 ), slp_node,  1 );

  int farm_node = faDDAQfarm_node();
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 4 ), farm_node - 50, 1 );
  
  //====================================================================
  //   raw event size
  //====================================================================  
  
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 6 ), float( faDEVTlength() ) / 1000, 1 ); // [kB]

  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 101 ), float( ArteTable<DSVD>::size()*sizeof( DSVD ) )/1000, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 102 ), float( ArteTable<DITR>::size()*sizeof( DITR ) )/1000, 1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 104 ), float( ArteTable<DOTR>::size()*sizeof( DOTR ) )/1000, 1 );

  //====================================================================
  //   hit occupancy
  //====================================================================  

  // hits per detector
  int hitsVDS  = 0;
  int hitsOTR  = 0;
  int hitsRICH = 0;
  for( ArteTableIterator<HITB> hitb; hitb.valid(); hitb.next() ){ 
    ArtePointer<GEDE> gede( hitb->gede ); 
    if( gede->cmp == 1 ) hitsVDS++;
    if( gede->cmp == 2 ) hitsOTR++;
    if( gede->cmp == 5 ) hitsRICH++;
  }

  // histos
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 201 ), hitsVDS,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 204 ), hitsOTR,  1 );
  rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 205 ), hitsRICH, 1 );  

  ARTE_mark( "DQ_FARM_hit_evnt 1", 0 );

  return 0;
}
