
// *****************************************************************************
//
// source:    faStop.C
//
// type:      source code
//
// created:    7 Jul 1997
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    fastop_()
//
// input:     void
// return:    int  ARTE_error_code
//
// called by: ATSTOP
//
// purporse:  termination of farm stuff called for FARM mode
//
// changes:
// 25 Nov 1998 (AG) stop rpm
// 27 Nov 1998 (AG) multiple file input
// 01 Dec 1998 (AG) daq format logging
// 11 Jan 1999 (AG) includes
// 12 Jan 1999 (AG) removed emg status flag
// 14 Jan 1999 (AG) removed hbook filling
// 21 Jan 1999 (AG) more printout
// 02 Feb 1999 (AG) halt rpm
// 23 Feb 1999 (AG) removed faUtil.hh
// 25 Feb 1999 (AG) use grr error logger
// 11 Mar 1999 (AG) polishing
// 01 Jun 1999 (AG) put includes
// 09 Sep 1999 (AG) always use cout
// 08 Oct 1999 (AG) write to log file
// 18 Oct 1999 (AG) polishing
// 03 Nov 1999 (AG) add process id to logfile
// 11 Nov 1999 (AG) improve log
// 19 Nov 1999 (AG) polishing
// 24 Nov 1999 (AG) use ARTE_error()
// 29 Feb 2000 (AG) linux -> __linux
// 14 Mar 2000 (AG) improve writeout
// 27 Apr 2000 (AG) use ARTE_mark()
// 28 Apr 2000 (AG) use ARTE_log()
// 15 May 2000 (AG) polish printout
// 23 Jun 2000 (AG) add multi timer stop
// 28 Jun 2000 (AG) commented faMultiTimer() for the time being
// 12 Jul 2000 (AG) removed MultiTimer
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ONLINE inludes
#include "rpm.h"
#include "rhp.h"
#include "grr.h"

// ARTE includes
#include "farm/fa.hh"
#include "farm/farm.hh"

// externals
extern int  nArg;   // line command arguments
extern char **vArg; // line command arguments

extern int  FARM;     // FARM mode
extern int  FAON;     // FARM online mode
extern char *DBNAME;  // database name
extern int  NOSPAR;   // sparsification
extern int  RPMINIT;  // rpm/rhp initialized
extern int  FAGRR;    // grr error logger     
extern char *PARNAME; // emg partition
extern int  SEG_NUM;  // emg partitions
extern int  SEG_LEN;  // emg partition size
extern char *LOGFILE; // log-file

//
// program code
//
extern "C" int
fastop_()
{
  // show up
  ARTE_mark( "faStop", 0 );
  
  //
  // farm mode
  //
  int rc = ARTE_GOOD;    
  if( FARM ){
    // emg
    if( FAON ) rc = faStopEmg();
  }
  
  //
  // grr error message
  //
  ARTE_error( GRR_SINFO, "STOP: faStop DONE.", 0 );

  //
  // multi/rhp/rpm
  //
  if( RPMINIT ){

    if( ( rc = rhp_halt() ) == GRR_OK ){
      cout << "STOP: rhp_halt" << endl;
    }
    else {
      cout << "STOP: ERROR: rhp_halt rc=" << rc << endl;
    }

    if( ( rc = rpm_halt() ) == GRR_OK ){
      cout << "STOP: rpm_halt" << endl;
    }
    else {
      cout << "STOP: ERROR: rpm_halt rc=" << rc << endl;
    }
  }

  //
  // write to logfile
  //
  ARTE_log( LOGFILE, "faStop" );

  // return
  return rc;
}
