
      SUBROUTINE MEDMIX(AM,ZM,WM,DENS,NM,AEFF,ZEFF,RADEFF,ABSEFF)

************************************************************************
* Author :    Siegmund Nowak   02/08/96
*
* Description: calculates A_MEAN,Z_MEAN,Radlen and Absorption length
*              for mixtures
*
* Called from:
*
* External references: GEANT functions  GFCOUL,GHSIGM,GXSI
*                      (no initialization of GEANT needed)
*
* Argument description
* AM       vector of atomic numbers of  mixture components       input
* ZM       vector of charges of mixture components               input
* WM       vector of weights of mixture components        input/output
* DENS     density of the mixture                                input
* AEFF     effective atomic number of the mixture               output
* ZEFF     effective charge of the mixture                      output
* RADEFF   radiation length of the mixture                      output
* ABSEFF   absorption length  of the mixture                    output
*
* This routine is taken from GEANT routines GSMIXT and GHMIX          SN
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
      INTEGER NM,INM,I,ITER
      REAL AM(NM),ZM(NM),WM(NM),DENS,AEFF,ZEFF,RADEFF,ABSEFF
      REAL AMOL,ZMOL,RADINV,ZC,ALZ,XINV
      REAL PINT,AINV,WTOT,PNEW,POLD,DP,DA,DAOLD,AHEFF
      REAL GFCOUL,GHSIGM,GXSI
      REAL ALR2AV, AL183
      DATA ALR2AV /1.39621E-03/, AL183 /5.20948/
#include "arte/Arte.ff"

C             Compute proportion by weigths in the compound
C
      INM = IABS(NM)
      IF(NM.LT.0) THEN
         AMOL   = 0.
         ZMOL   = 0.
         DO 10 I= 1,INM
         AMOL   = AMOL + WM(I)*AM(I)
         ZMOL   = ZMOL + WM(I)*ZM(I)
   10    CONTINUE
         DO 20 I= 1,INM
         WM(I)= WM(I)*AM(I) / AMOL
   20    CONTINUE
      ENDIF

C     Compute effective mixture parameters

C     Radiation length
      AEFF   = 0.
      ZEFF   = 0.
      RADINV = 0.
      DO 40 I = 1,INM
         AEFF   = AEFF + WM(I)*AM(I)
         ZEFF   = ZEFF + WM(I)*ZM(I)
         ZC     = ZM(I)
         ALZ    = LOG(ZC)/3.
         XINV   = ZC*(ZC+GXSI(ZC))*(AL183-ALZ-GFCOUL(ZC))/AM(I)
         RADINV = RADINV + WM(I)*XINV
   40 CONTINUE
      RADINV = ALR2AV * DENS * RADINV
      RADEFF = 1. / RADINV

C     Absorbtion length
      PINT   = 0.
      AINV   = 0.
      WTOT   = 0.
      DO 50 I = 1, INM
         PINT   = PINT + WM(I) * GHSIGM(5.,8, AM(I)) / AM(I)
         AINV   = AINV + WM(I) / AM(I)
         WTOT   = WTOT + WM(I)
  50  CONTINUE
      IF(ABS(WTOT-1.) .GT. 0.01 ) THEN
         C78 = 'MEDMIX: sum of mixture  weights do not add up to 1'
         CALL CH78BF (WTOT)
         CALL ME78EC(8388900)
      ENDIF
C
C         Work out AHEFF which gives PINT for 5 GeV/c pion.
C         (This is a short Newtons method loop.)
C
      AHEFF   = 1. / AINV
      PNEW   = GHSIGM ( 5., 8, AHEFF ) / AHEFF
      DA     = 1.
      ITER   = 0
   60 CONTINUE
      ITER   = ITER + 1
      AHEFF   = AHEFF + DA
      POLD   = PNEW
      DAOLD  = DA
      PNEW   = GHSIGM ( 5., 8, AHEFF ) / AHEFF
      DP     = PNEW - POLD
      DA     = (PINT - PNEW ) * DAOLD / DP
      IF((ITER .GT. 1 ) .AND. (ABS(DA) .GT. ABS(DAOLD))) THEN
        C78 = 'MEDMIX: no convergence for hadronic interaction mixture'
        CALL ME78EC(8388901)
      ENDIF
      IF(ABS(DA) .GT. 0.01 ) GO TO 60

      ABSEFF=10000.*AHEFF/(6.022*DENS*GHSIGM(5.,8,AHEFF))

      RETURN
      END

************************************************************************
* End of MEDMIX
************************************************************************
