/* Usage: 
       ...
       // x is of any type X
       write_binary(x, cout); // copies sizeof(X) bytes of x to cout
       read_binary(x, cin);   // reads  sizeof(X) bytes of x from cin
   This works for any type.

   Such binary format is usually economical, but computer architecture
   dependent.
   */

// Author V.Balagura (balagura@mail.desy.de)   (05.08.98).

#ifndef _io_binary_hh
#define _io_binary_hh 1

#include <iostream.h>
// #include "ptset/global_error.hh"

// --------------- read/write T in binary format ---------------
template <class T>
inline void write_binary(const T& x, ostream& os) {
  signed char* pc = (signed char*) &x;
  for (unsigned int i=0; i<sizeof(T); ++i) {
    if (!os.put(*pc++)) {
      //      global_error()("can not write value in binary format");
      return;
    }
  }
}
template <class T>
inline void read_binary(T& x, istream& is) {
  signed char* pc = (signed char*) &x;
  for (unsigned int i=0; i<sizeof(T); ++i) {
    if (!is.get(*pc++)) {
      //      global_error()("can not read value in binary format");
      return;
    }
  }
}

#endif
