// 
//  Author:  Rainer Mankel                     Created:  01-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerLayPl

// contains the GEDE-related information of a domain layer.
// each GEDE plane which contributes to a domain layer is represented
// by a layer plane, which stores the valid u range and the contributing
// sub layers. LayPl contains also a list of pointers to exclusion ranges.

#ifndef RANGER_LAYPL_HH
#define RANGER_LAYPL_HH

#include <list.h>
#include "arte/arte.hh"

class rangerDomLay;
class rangerExclusionRange;

class rangerLayPl {

public:
  rangerLayPl(){};
  rangerLayPl(rangerDomLay* dl,ArtePointer<GEDE> ge,
	      const int subGLay1,const int subGLay2);
  ~rangerLayPl(){};
  ArtePointer<GEDE> ge() { return gede; };           // accessors
  int subgl1() { return subGL1;};                    //
  int subgl2() { return subGL2;};                    //
  double umin() { return uMin;};                     //
  double umax() { return uMax;};                     //
  int excludeRange(double uExclMin,double uExclMax,
		   int subgl1=1,int subgl2=99); // install exclusion range
  int clearExclusionRanges();      // delete all exclusion ranges
  void dump(const int level = 1);
private:

  rangerDomLay* dLay;  // associated domain layer
  ArtePointer<GEDE> gede; // associated GEDE plane
  int subGL1;          // sublayer range (first/last)
  int subGL2;
  double uMin;         // range of projected coordinate
  double uMax;
  list<rangerExclusionRange*> exrs; 
};

#endif
