
#ifndef _ECLASS_HH
#define _ECLASS_HH

// *****************************************************************************
//
// source:    eclass.hh
//
// type:      header file
//
// created:   08 May 2000
//    
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// purpose:   define event classification bits (1-32)
//
// hints:     evhd->flagdst is defined as 'int'.
//
// changes:
// 09 May 2000 (AG) introduce class JPSIEE and JPSIMM
// 23 May 2000 (AG) introduce class RNDTRG
// 25 May 2000 (AG) introduce class FLTTRG
// 29 May 2000 (AG) introduce single high-ET ECAL trigger
//  5 Jun 2000 (AG) change ECLASS_FLTTRG to bit 30
//  7 Jun 2000 (AG) leave upper 16 bits open for users
// 14 Jun 2000 (AG) introduce ECLASS_FCNC
// 23 Jun 2000 (AG) introduce names for classes
// 25 Jun 2000 (AG) correct names
// 28 Jun 2000 (AG) added more classes
//
// *****************************************************************************

// physical classes (reserved)
#define ECLASS_JPSIEE  0x1    // bit= 1 J/Psi -> e+ e-
#define ECLASS_JPSIMM  0x2    // bit= 2 J/Psi -> mu+ mu-
#define ECLASS_CAL1HT  0x4    // bit= 3 single high-ET ECAL cluster trigger
#define ECLASS_FCNC    0x8    // bit= 4 FCNC event
#define ECLASS_5       0x10   // bit= 5
#define ECLASS_6       0x20   // bit= 6
#define ECLASS_7       0x40   // bit= 7
#define ECLASS_8       0x80   // bit= 8
#define ECLASS_9       0x100  // bit= 9
#define ECLASS_10      0x200  // bit=10
#define ECLASS_11      0x400  // bit=11
#define ECLASS_12      0x800  // bit=12
#define ECLASS_13      0x1000 // bit=13
#define ECLASS_14      0x2000 // bit=14
#define ECLASS_FLTTRG  0x4000 // bit=15 FLT trigger    (FCS_tcode = 0 || 8)
#define ECLASS_RNDTRG  0x8000 // bit=16 random trigger (FCS_tcode = 8 || 9)

#define ECLASS_NAME_1  "JPSIEE"
#define ECLASS_NAME_2  "JPSIMM"
#define ECLASS_NAME_3  "CAL1HT"
#define ECLASS_NAME_4  "FCNC"
#define ECLASS_NAME_5  "UNDEF"
#define ECLASS_NAME_6  "UNDEF"
#define ECLASS_NAME_7  "UNDEF"
#define ECLASS_NAME_8  "UNDEF"
#define ECLASS_NAME_9  "UNDEF"
#define ECLASS_NAME_10 "UNDEF"
#define ECLASS_NAME_11 "UNDEF"
#define ECLASS_NAME_12 "UNDEF"
#define ECLASS_NAME_13 "UNDEF"
#define ECLASS_NAME_14 "UNDEF"
#define ECLASS_NAME_15 "FLTTRG"
#define ECLASS_NAME_16 "RNDTRG"

// user defined classes
#define ECLASS_USER1  0x10000    // bit=17
#define ECLASS_USER2  0x20000    // bit=18
#define ECLASS_USER3  0x40000    // bit=19
#define ECLASS_USER4  0x80000    // bit=20
#define ECLASS_USER5  0x100000   // bit=21
#define ECLASS_USER6  0x200000   // bit=22
#define ECLASS_USER7  0x400000   // bit=23
#define ECLASS_USER8  0x800000   // bit=24
#define ECLASS_USER9  0x1000000  // bit=25
#define ECLASS_USER10 0x2000000  // bit=26
#define ECLASS_USER11 0x4000000  // bit=27
#define ECLASS_USER12 0x8000000  // bit=28
#define ECLASS_USER13 0x10000000 // bit=29
#define ECLASS_USER14 0x20000000 // bit=30
#define ECLASS_USER15 0x40000000 // bit=31
#define ECLASS_USER16 0x80000000 // bit=32

#define ECLASS_NAME_17  "USER1"
#define ECLASS_NAME_18  "USER2"
#define ECLASS_NAME_19  "USER3"
#define ECLASS_NAME_20  "USER4"
#define ECLASS_NAME_21  "USER5"
#define ECLASS_NAME_22  "USER6"
#define ECLASS_NAME_23  "USER7"
#define ECLASS_NAME_24  "USER8"
#define ECLASS_NAME_25  "USER9"
#define ECLASS_NAME_26  "USER10"
#define ECLASS_NAME_27  "USER11"
#define ECLASS_NAME_28  "USER12"
#define ECLASS_NAME_29  "USER13"
#define ECLASS_NAME_30  "USER14"
#define ECLASS_NAME_31  "USER15"
#define ECLASS_NAME_32  "USER16"

// prototypes
char *rcClassName( int bit );

#endif
