*--   Author :    Vitaly Eiges   07/11/99

      SUBROUTINE MDBEND (r_vds, tx_bend, ptot, x_mu1) 

***************************************************************
*
* Description: iterations to swim track from VDS (x,y,z,tx,ty) 
*              to "after magnet" position with known delta(T_x)
*              bending.
*              Returns "ptot" estimated from this constraint
*              r_vds(5) - vds track coordinates
*              tx_bend - tx slope after magnet
*              It's also posible that program will transport
*              the track with determined momentum to X in MU1 !
*
*
*              x_mu1 - X in MU1, track is transported to Z=1620.
*
* Changes:
* 07 Jul 2000 (AG) called source rcmdbend and commented hbook
*
***************************************************************

      IMPLICIT NONE

      real r_vds(5) ! x,y,z,tx,ty
      real tx_bend, ptot, x_mu1

      real xv, yv, zv
      real xr, yr, zr
      real txr, tyr, txv, tyv
      real pxz, p_tot
      real p_vds(5), p_rich(5)   ! x,y,tx,ty,q/p

      real tx_accuracy, p_step
      integer n_iter, max_iter

      logical frst

      real z_after_magnet, pt_kick, z_mu1

      real dtx ! tx bending difference

      data tx_accuracy / 0.0003 /
      data p_step / 0.1 /
      data max_iter / 50 /
      data z_after_magnet / 1200. /
      data pt_kick / 0.64 /
      data z_mu1 / 1620. /
      data frst  /.true./

      if( frst ) then
CAG         call HBOOK1(1972, 'dtx initial', 100, -0.02, 0.02, 0.)
         frst = .false. 
      endif
      
      xv = r_vds(1)
      yv = r_vds(2)
      zv = r_vds(3)
      txv = r_vds(4)
      tyv = r_vds(5)

      p_vds(1) = xv
      p_vds(2) = yv
      p_vds(3) = txv
      p_vds(4) = tyv

      dtx = tx_bend - txv
      pxz = pt_kick/dtx         ! initial approximation
C      print*,'initial pxz= ',pxz

      ptot = pxz
      p_vds(5) = 1./ptot

      call rgauxsvm(zv, p_vds, z_after_magnet, p_rich)
      dtx = tx_bend - p_rich(3)
CAG      call hfill(1972, dtx, 0., 1.)
c      print*,'init dtx= ',dtx

      n_iter = 0

      do while (abs(dtx).gt.tx_accuracy)
         n_iter = n_iter + 1
c         print*,'iter, p, dtx', n_iter, ptot, dtx
         if (dtx.lt.0.) ptot = ptot + p_step
         if (dtx.gt.0.) ptot = ptot - p_step
         p_vds(5) = 1./ptot
         call rgauxsvm(zv, p_vds, z_after_magnet, p_rich)
         dtx = tx_bend - p_rich(3)
         if (n_iter.gt.max_iter) goto 900
      enddo

 900  continue

c      print*,'final iter, p, dtx', n_iter, ptot, dtx

      p_vds(5) = 1./ptot
      call rgauxsvm(zv, p_vds, z_mu1, p_rich)
      x_mu1 = p_rich(1)
c      print*,'x_mu1 = ',x_mu1

      end
