!-----------------------------------------------------------------------
! riter - fortran version of 
! M. Staric, jan-2000
!
!-----------------------------------------------------------------------


      SUBROUTINE RITER(IER)

      INTEGER IER

#include "riter/RIC_PAR.fi"
#include "riter/RIC_TRA.fi"
#include "riter/RIC_PHOT.fi"

      LOGICAL START /.TRUE./
      LOGICAL RITER_ENABLE /.FALSE./
      SAVE START, RITER_ENABLE

      IER=-2
      IF(START) THEN
         START=.FALSE.
         CALL RITER_INIT(IER)
         IF(IER.NE.0) RETURN
         RITER_ENABLE=.TRUE.
      ENDIF
      IF(.NOT.RITER_ENABLE) RETURN

      CALL RESET_RING_TAB(BIN_SIZE)
      CALL GET_TRACKS(IER)
      IF(IER.NE.0) RETURN
      CALL GET_PHOTONS(IER)
      IF(IER.NE.0) RETURN

      DO ITRA=1,NTRA
         CALL TRACK_IN_RICH(ITRA)
         IF(RECO_FLAG(ITRA).EQ.0) GOTO 1
         CALL SIGNAL_ESTIMATE(ITRA)
         IF(RECO_FLAG(ITRA).EQ.0) GOTO 1
         CALL SET_RICWIN(ITRA)
         IF(RECO_FLAG(ITRA).EQ.0) GOTO 1
         DO IFOT=1,NFOT
            IF(UTEZ(IFOT).EQ.0) GOTO 11
            CALL CALC_THCER(RF(1,IFOT),SIG_F(IFOT),TH_CER,FI_CER,SIG)
            IF(SIG.EQ.0) GOTO 11
            UT=UTEZ(IFOT)
            CALL PUT_RING_TAB(ITRA,IFOT,SIG,TH_CER,FI_CER,UT,IER)
            IF(IER.NE.0) RETURN
 11         CONTINUE
         ENDDO
 1       CONTINUE
      ENDDO

      DO ITER=1,NITER
         CALL RING_ITER(WDIFF)
      ENDDO

      CALL BGR_ESTIMATE(NITER)
      CALL LIKELIHOOD(NITER)
      CALL RITER_TO_RTRA

      RETURN
      END


!-----------------------------------------------------------------------
! riter initialisation
! M. staric, jan-2000
!
!-----------------------------------------------------------------------

      SUBROUTINE RITER_INIT(IER)

      INTEGER IER

#include "riter/RIC_PAR.fi"
      DATA N_ZERO /4.2/
      DATA REF_IND /0./         ! if 0, get it from RITH
      DATA SIG_M16 /.7E-3/
      DATA SIG_M4 /1.05E-3/
      DATA ETA /1./
      DATA THC_LIM /70.E-3/
      DATA BIN_SIZE /.5E-3/
      DATA NITER /10/
      DATA SWIN_SIZE /6./
      DATA DWIN_SIZE /3./
      DATA HISTO_ON  /0/

#include "riter/RIC_VES.fi"
      DATA Z_IN /8650./
      DATA Z_OUT /11600./
      DATA ENTW_A /2200./, ENTW_B /1400./

      CALL DQ_RICH_SWITCH_RITER( HISTO_ON )
      CALL LOAD_GEO(IER)
      IF(IER.NE.0) RETURN
      CALL TO_VIRT_POS
      CALL MEALLP('RITER: initialisation finished')
      CALL MEALLP('-------------------------------------------')
      RETURN
      END


!-----------------------------------------------------------------------
! Modification of parameters in /RIC_PAR/ with kuip interface
! M. Staric, jul-2000
!-----------------------------------------------------------------------


      SUBROUTINE RITER_SET_NZERO

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(N_ZERO)
      PRINT*,'RITER: N0 - fig. of merit for RICH [mm^-1] = ',N_ZERO
      RETURN
      END


      SUBROUTINE RITER_SET_REFIND

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(REF_IND)
      PRINT*,'RITER: refractive index = ',REF_IND
      RETURN
      END


      SUBROUTINE RITER_SET_SIGM16

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(SIG_M16)
      PRINT*,'RITER: angular sigma for M16 region [rad] = ',SIG_M16
      RETURN
      END


      SUBROUTINE RITER_SET_SIGM4

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(SIG_M4)
      PRINT*,'RITER: angular sigma for M4 region [rad] = ',SIG_M4
      RETURN
      END


      SUBROUTINE RITER_SET_ETA

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(ETA)
      PRINT*,'RITER: signal normalisation factor = ',ETA
      RETURN
      END


      SUBROUTINE RITER_SET_THCLIM

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(THC_LIM)
      PRINT*,'RITER: limit for Cerenkov angle [rad] = ',THC_LIM
      RETURN
      END



      SUBROUTINE RITER_SET_BIN

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(BIN_SIZE)
      PRINT*,'RITER: bin size used for ring_iter [rad] = ',BIN_SIZE
      RETURN
      END


      SUBROUTINE RITER_SET_NITER

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETI(NITER)
      PRINT*,'RITER: number of iterations = ',NITER
      RETURN
      END


      SUBROUTINE RITER_SET_SWIN

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(SWIN_SIZE)
      PRINT*,'RITER: signal window size in num. of SIGMAs = ',SWIN_SIZE
      RETURN
      END


      SUBROUTINE RITER_SET_DWIN

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      CALL KUGETR(DWIN_SIZE)
      PRINT*,'RITER: wide window size in number of SWINs = ',DWIN_SIZE
      RETURN
      END


      SUBROUTINE RITER_SHOWPAR

      IMPLICIT NONE
#include "riter/RIC_PAR.fi"

      PRINT*,'RITER: Current settings of parameters in /RIC_PAR/:'
      PRINT*
      PRINT*,'  NZERO     ',N_ZERO
      PRINT*,'  REFIND    ',REF_IND
      PRINT*,'  SIGM16    ',SIG_M16
      PRINT*,'  SIGM4     ',SIG_M4
      PRINT*,'  ETA       ',ETA
      PRINT*,'  THCLIM    ',THC_LIM
      PRINT*,'  BIN       ',BIN_SIZE
      PRINT*,'  NITER     ',NITER
      PRINT*,'  SWIN      ',SWIN_SIZE
      PRINT*,'  DWIN      ',DWIN_SIZE
      PRINT*,'  rhp flag  ',HISTO_ON
      PRINT*

      RETURN
      END
