*CMZ :          31/10/96  07.10.14  by  T. Oest
*-- Author :    T. Oest   29/10/96

      SUBROUTINE V0FITS(TRK1, TRK2, ZSTART, PAR, DTX, DTY, IRET)

************************************************************************
*
* Description: returns start values for parameters and
*              covariance matrix
*
* Called from:
*
* External references:
*
* Argument in/out description
* TRK1     in     first track
* TRK2     in     second track
* ZSTART   in     z position of vertex
* PAR      out    vertex, track parameters at START
*                 (ZSTART,x,y,tx1,ty1,1/p1,tx2,ty2,1/p2)
* DTX      out    tx(vertex)-tx(momentum)
* DTY      out    ty(vertex)-ty(momentum)
* IRET     out    = 0   ok
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"

      INTEGER          TRK1, TRK2, IRET
      DOUBLE PRECISION ZSTART, PAR(9), DTX, DTY

      LOGICAL          SCATT
      INTEGER          I, J, IERR1, IERR2
      DOUBLE PRECISION PTRK1(5), PTRK2(5), VTRK1(5,5), VTRK2(5,5)
      DOUBLE PRECISION V1(2,2), V2(2,2), W1(2,2), W2(2,2)
      DOUBLE PRECISION WSUM(2,2), VSUM(2,2), RSUM(2)
      DOUBLE PRECISION PX, PY, PZ(2)

#include "arte/Arte.ff"

      DATA SCATT /.TRUE./
C-----------------------------------------------------------------------
       IRET = 0
C-----------------------------------------------------------------
C  swim tracks to ZSTART
C-----------------------------------------------------------------
       CALL V0SWIM (SCATT, TRK1, ZSTART, PTRK1, VTRK1, IERR1)
       CALL V0SWIM (SCATT, TRK2, ZSTART, PTRK2, VTRK2, IERR2)
       IF (IERR1.NE.0 .OR. IERR2.NE.0) THEN
        IRET  = -1
        GOTO 9000
       ENDIF


C-----------------------------------------------------------------
C  weighted mean for x,y
C-----------------------------------------------------------------
       DO I=1,2
       DO J=1,2
        V1(I,J) = VTRK1(I,J)
        V2(I,J) = VTRK2(I,J)
       ENDDO
       ENDDO
       CALL V0INV2(V1,W1,IERR1)
       CALL V0INV2(V2,W2,IERR2)
       IF (IERR1.NE.0 .OR. IERR2.NE.0) THEN
        IRET  = -2
        GOTO 9000
       ENDIF

       DO I=1,2
       DO J=1,2
        WSUM(I,J) = W1(I,J) + W2(I,J)
       ENDDO
       ENDDO

       CALL V0INV2(WSUM,VSUM,IERR1)
       IF (IERR1.NE.0) THEN
        IRET  = -3
        GOTO 9000
       ENDIF

       DO I=1,2
        RSUM(I) = 0.0
       DO J=1,2
        RSUM(I) = RSUM(I) + W1(I,J)*PTRK1(J) + W2(I,J)*PTRK2(J)
       ENDDO
       ENDDO

       DO I=1,2
        PAR(I+1) = 0.0
       DO J=1,2
        PAR(I+1) = PAR(I+1) + VSUM(I,J) * RSUM(J)
       ENDDO
       ENDDO
C-----------------------------------------------------------------
C  rest of parameters
C-----------------------------------------------------------------
       PAR(1) = ZSTART

       PAR(4) = PTRK1(3) - VTRK1(3,1)/VTRK1(1,1)*(PTRK1(1)-PAR(2))
     +                   - VTRK1(3,2)/VTRK1(2,2)*(PTRK1(2)-PAR(3))
       PAR(5) = PTRK1(4) - VTRK1(4,1)/VTRK1(1,1)*(PTRK1(1)-PAR(2))
     +                   - VTRK1(4,2)/VTRK1(2,2)*(PTRK1(2)-PAR(3))
       PAR(6) = PTRK1(5) - VTRK1(5,1)/VTRK1(1,1)*(PTRK1(1)-PAR(2))
     +                   - VTRK1(5,2)/VTRK1(2,2)*(PTRK1(2)-PAR(3))
       PAR(7) = PTRK2(3) - VTRK2(3,1)/VTRK2(1,1)*(PTRK2(1)-PAR(2))
     +                   - VTRK2(3,2)/VTRK2(2,2)*(PTRK2(2)-PAR(3))
       PAR(8) = PTRK2(4) - VTRK2(4,1)/VTRK2(1,1)*(PTRK2(1)-PAR(2))
     +                   - VTRK2(4,2)/VTRK2(2,2)*(PTRK2(2)-PAR(3))
       PAR(9) = PTRK2(5) - VTRK2(5,1)/VTRK2(1,1)*(PTRK2(1)-PAR(2))
     +                   - VTRK2(5,2)/VTRK2(2,2)*(PTRK2(2)-PAR(3))

C-----------------------------------------------------------------
C  dtx, dty
C-----------------------------------------------------------------
      PZ(1)   = SQRT(1.+PAR(4)**2+PAR(5)**2)/PAR(6)
      PZ(2)   = SQRT(1.+PAR(7)**2+PAR(8)**2)/PAR(9)
      PX      = PZ(1) * PAR(4) + PZ(2) * PAR(7)
      PY      = PZ(1) * PAR(5) + PZ(2) * PAR(8)

      DTX = PAR(2)/PAR(1) - PX/(PZ(1)+PZ(2))
      DTY = PAR(3)/PAR(1) - PY/(PZ(1)+PZ(2))

C-----------------------------------------------------------------
C  the end
C-----------------------------------------------------------------
 9000 END

************************************************************************
* End of V0FITS
************************************************************************


